
use strict;
use Data::Dumper;
use IO::Handle;
use IO::Select;
use Symbol qw(qualify_to_ref);

use vars  qw( $meta $required $missing $priority $PERL $PERL_VERSION $SLOWDOWN );

unless (    ( $required = do 'sbin/required/modules'            )
	 && ( $meta     = do 'sbin/required/modules_meta'       ) )
  {
    print Dumper $!;
    print Dumper $@;
  }

require CPAN;

$PERL_VERSION =  $];

$PERL     = $ARGV[0];
$SLOWDOWN = $ARGV[1] || 0;

print "you are running Perl $PERL, version $PERL_VERSION \n";

my ($req_mod,$mod);

my @all = CPAN::Shell->expand( "Module", "/./" );

print "Looking for required perl modules:\n";
REQ:
    foreach $req_mod ( keys %{ $required } ) {
        foreach $mod ( @all ) {
            if ( $mod->id eq $req_mod ) {
                if ( $mod->inst_file ) {
                    print " + $req_mod \n";
                    next REQ;
                } else {
                    $missing->{ $req_mod } = $mod;
                }
            }
        }
    }

# case of all dependencies meet
my @keys = keys %{ $missing };
if ( $#keys == -1 ) {
    print "All modules found!\n";
    exit;
}

unless ( defined( $ENV{'FTP_PASSIVE'} ) && $ENV{'FTP_PASSIVE'} eq 1) {
  $ENV{'FTP_PASSIVE'} = 1;
  print "\nSetting FTP Passive mode.\n";
}

foreach ( keys %{ $meta } ) {
    $priority->{ $_ } = $meta->{ $_ }{priority}
      if $meta->{ $_ }{priority};
}

PRI:
    foreach my $pri ( keys %{ $priority } ) {
        foreach $mod ( keys %{ $missing } ) {
            if ( $pri eq $mod ) {
                delete( $missing->{ $mod } );
                next PRI;
            }
        }
        #print "removing $pri from priority \n";
        delete( %{ $priority }->{ $pri } );
    }

my @tmp = keys %{ $priority };
unless ($#tmp == -1) {
    print "\nPriority missing modules (to be installed):\n";
        foreach $mod ( keys %{ $priority } ) {
            print " - $mod \n";
        }
}

@tmp = keys %{ $missing };
unless ($#tmp == -1) {
    print "\nOther missing modules (to be installed):\n";
    foreach $mod ( keys %{ $missing } ) {
        print " - $mod \n";
    }
}

foreach $mod ( sort { $priority->{$a} <=> $priority->{$b} }
	       keys %{ $priority } ) {
    &install_module( $mod,"y" );
}

foreach $mod ( keys %{ $missing } ) {
    &install_module( $mod,"y" );
}

##----------------------
# Install a CPAN module
##----------------------
sub install_module {
    my ( $module, $default, $force ) = @_;

    unless ( $> == 0 ) {
        print "\nWARNING: You need root privileges to install required $module module. \n";
        print "   - " . $meta->{ $module }{role} . "\n";
        return undef;
    }

    my ($force,$test_failure);
    my $force_extra = "";
    if ($meta->{ $module }{rough}{brutal_force}) {
	$force = 1;
	$force_extra = " by brutal force (unsuitable for failure rate testing)"
    } elsif ( $meta->{ $module }{rough}{failure_rate} ) {
	$test_failure = 1;
	$force_extra = " with failure testing (known to fail some tests)"
    }


    print  "\n======================================================\n";
    print  "  + Installing module $module $force_extra \n";
    printf "  + Role: %s \n", $meta->{ $module }{role};
    print  "========================================================\n\n";

    &slowdown;

    if ( $force ) {
	CPAN::Shell->force( 'install', $module );
	return;
    }

    if ( $test_failure ) {

	my $failure_threshold = $meta->{ $module }{rough}{failure_rate};

        # fork, capture stdout and stderr
        my $command = "$PERL -MCPAN -e 'install \"$module\"'";
        my $pid = open ( OUT, "$command 2>&1 |" )
            or die "Can't run program: $!\n";

        my $sucess_rate;

        print "forked: \n  PID: $pid \n  command: $command\n\n";

	&slowdown;

        while (<OUT>) {
            print;
            if ( /^([A-Za-z]+) ([0-9]+\/[0-9]+) test scripts, ([0-9.]+)% okay. ([0-9]+\/[0-9]+) subtests failed, ([0-9.]+)% okay./ ) {
                print "JUNK = $1 \n STUFF = $2 \n AND $3 \n AND $4 \n FAILED_TEST% $5\n";
                $sucess_rate = $5;
            }
        }

        if ( int($sucess_rate) > $failure_threshold ) {
	    print  "\n======================================================\n";
            print " + WARNING: some tests for $module failed. however, $sucess_rate% has succeded. \n";
            print " + WARNING: running forceful install ('cpan force install $module')\n";
	    print  "\n======================================================\n\n";
	    &slowdown;
	    CPAN::Shell->force( 'install', $module );
        }

        close OUT;

    } else {

        CPAN::Shell->install( $module );

    }

}

sub slowdown {
    return unless $SLOWDOWN;
    print "\n(sleeping $SLOWDOWN)\n\n";
    sleep $SLOWDOWN;
    return;
}













